using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Practices.EnterpriseLibrary.Configuration.Design;
using System.ComponentModel.Design;
using EnterpriseLibraryExtensions;
using System.ComponentModel;
using System.IO;

namespace EnterpriseLibraryExtensions.ConfigurationConstsWriter
{
    public class ConfigurationConstsWriterManager : ConfigurationDesignManager
    {
        public override void Register(IServiceProvider serviceProvider)
        {
            ConfigurationConstsCommandRegistrar.Register(serviceProvider);
            ServiceHelper.GetUIHierarchyService(serviceProvider).SelectedHierarchy.Saved += new EventHandler<HierarchySavedEventArgs>(SelectedHierarchy_Saved);
        }

        void SelectedHierarchy_Saved(object sender, HierarchySavedEventArgs e)
        {
            foreach(ConfigurationConstsWriterNode constWriter in e.UIHierarchy.FindNodesByType(typeof(ConfigurationConstsWriterNode)))
            {
                ConfigurationConstsWriterSettings settings = ConfigurationConstsWriterSettingsBuilder.Build(constWriter.Site, constWriter);
                ConfigurationConstsWriter.Write(e.UIHierarchy.FindNodeByType(typeof(ConfigurationConstsWriterNode)), settings, Path.GetDirectoryName(e.UIHierarchy.RootNode.ConfigurationFile));
            }
        }

        protected override void OpenCore(IServiceProvider serviceProvider, ConfigurationApplicationNode rootNode, System.Configuration.ConfigurationSection section)
        {
            if (section != null)
            {
                rootNode.AddNode(ConfigurationConstsWriterNodeBuilder.Build(serviceProvider, section as ConfigurationConstsWriterSettings));
            }
        }

        protected override ConfigurationSectionInfo GetConfigurationSectionInfo(IServiceProvider serviceProvider)
        {
            ConfigurationNode rootNode = ServiceHelper.GetCurrentRootNode(serviceProvider);
            ConfigurationConstsWriterNode node = null;
            if (null != rootNode) node = rootNode.Hierarchy.FindNodeByType(rootNode, typeof(ConfigurationConstsWriterNode)) as ConfigurationConstsWriterNode;
            ConfigurationConstsWriterSettings configWriterSettings = null;
            if (node != null)
            {
                configWriterSettings = ConfigurationConstsWriterSettingsBuilder.Build(serviceProvider, node);
            }
            return new ConfigurationSectionInfo(node, configWriterSettings, ConfigurationConstsWriterSettings.SectionName);
        }
    }
}
