using System;
using System.Collections.Generic;
using System.Text;
using System.ComponentModel;
using Microsoft.Practices.EnterpriseLibrary.Configuration.Design;
using Microsoft.Practices.EnterpriseLibrary.Configuration.Design.Validation;
using EnterpriseLibraryExtensions.ConfigurationConstsWriter.Properties;
using System.Drawing.Design;
using System.Windows.Forms.Design;
using System.Reflection;

namespace EnterpriseLibraryExtensions.ConfigurationConstsWriter
{

    [Image(typeof(ConfigurationConstsWriterNode))]
    [SelectedImage(typeof(ConfigurationConstsWriterNode))]
    [ProvideProperty(ConstantNameProperty, typeof(ConfigurationNode))]
    //[NodeInformation(typeof(ConfigurationConstsWriterNode), "ConfigurationConstsWriterNodeInfoHtml", "ConfigurationConstsWriterNodeInfoTitle", typeof(Resources))]
    public class ConfigurationConstsWriterNode : ConfigurationNode, IExtenderProvider, ISitedConfigurationNodeSink
    {
        public const string ConstantNameProperty = "ConstantName";

        string _namespace;
        string _class;
        string _filePath;
        CodingLanguage _codingLanguage;
        IExtenderStorageService _extenderStorageService;
        ICurrentSelectionService _currentSelectionService;
        IUIService _uiService;


        public ConfigurationConstsWriterNode()
            :this(new ConfigurationConstsWriterSettings())
        {
        }

        public ConfigurationConstsWriterNode(ConfigurationConstsWriterSettings section)
        {
            Name = section.Name;
            _namespace = section.Namespace;
            _class = section.Class;
            _filePath = section.FilePath;
            _codingLanguage = section.CodingLanguage;
        }

        #region IExtenderProvider logic

        [DefaultValue(default(string))]
        [SRDescription("ConstantName", typeof(Resources))]
        public string GetConstantName(ConfigurationNode configNode)
        {
            return _extenderStorageService.GetExtendedPropertyValue<string>(this, ConstantNameProperty, configNode);
        }

        public void SetConstantName(ConfigurationNode configNode, string constantName)
        {
            _extenderStorageService.SetExtendedPropertyValue<string>(this, ConstantNameProperty, configNode, string.IsNullOrEmpty(constantName) ? null : constantName);
        }

        public bool CanExtend(object extendee)
        {
            if (extendee is ConfigurationConstsWriter) return false;
            if (extendee as ConfigurationNode == null) return false;
            if (((ConfigurationNode)extendee).Hierarchy != Hierarchy) return false;
            if (extendee is ConfigurationApplicationNode) return false;
            if (_currentSelectionService.CurrentlySelectedNode != extendee) return false;

            PropertyInfo nameProperty = extendee.GetType().GetProperty("Name");
            ReadOnlyAttribute[] readonlies = (ReadOnlyAttribute[]) nameProperty.GetCustomAttributes(typeof(ReadOnlyAttribute), true);
            if (readonlies.Length != 0 && readonlies[0].IsReadOnly) return false;

            BrowsableAttribute[] browsables = (BrowsableAttribute[]) nameProperty.GetCustomAttributes(typeof(BrowsableAttribute), true);
            if (browsables.Length != 0 && !browsables[0].Browsable) return false;

            
            return true;
            
        }

        #endregion

        [Required]
        [SRCategory("CategoryGeneral", typeof(Resources))]
        [SRDescription("NamespaceName", typeof(Resources))]
        public string NamespaceName
        {
            get { return _namespace; }
            set { _namespace = value; }
        }

        [Required]
        [SRCategory("CategoryGeneral", typeof(Resources))]
        [SRDescription("ClassName", typeof(Resources))]
        public string ClassName
        {
            get { return _class; }
            set { _class = value; }
        }

        [Required]
        [SRCategory("CategoryGeneral", typeof(Resources))]
        [SRDescription("FilePath", typeof(Resources))]
        [Editor(typeof(FileNameEditor), typeof(UITypeEditor))]
        public string FilePath
        {
            get { return _filePath; }
            set { _filePath = value; }
        }

        [Required]
        [SRCategory("CategoryGeneral", typeof(Resources))]
        [SRDescription("CodingLanguage", typeof(Resources))]
        public CodingLanguage CodingLanguage
        {
            get { return _codingLanguage; }
            set { _codingLanguage = value; }
        }

        #region ISitedConfigurationNodeSink Members

        public void Sited()
        {
            _extenderStorageService = (IExtenderStorageService) Site.GetService(typeof(IExtenderStorageService));
            _currentSelectionService = (ICurrentSelectionService)Site.GetService(typeof(ICurrentSelectionService));
            _uiService = (IUIService) Site.GetService(typeof(IUIService));
        }

        public void UnSited()
        {
        }

        #endregion
    }

    public enum CodingLanguage
    {
        CSharp,
        VB
    }
}
