using System;
using System.Collections.Generic;
using System.Text;
using System.ComponentModel.Design;
using Microsoft.Practices.EnterpriseLibrary.Configuration.Design;
using EnterpriseLibraryExtensions.ConfigurationConstsWriter.Properties;
using System.IO;
using System.Diagnostics;
using System.Threading;

namespace EnterpriseLibraryExtensions.ConfigurationConstsWriter
{
    class RegenerateConstFileVerb : IConfigDesignerVerb
    {
        #region IConfigDesignerVerb Members

        public List<DesignerVerb> VerbsForCofigurationNode(ConfigurationNode node)
        {
            List<DesignerVerb> verbs = new List<DesignerVerb>();
            ConfigurationConstsWriterNode _nodeToExtend = node as ConfigurationConstsWriterNode;
            if (_nodeToExtend != null)
                verbs.Add(new VerbContainer(_nodeToExtend).TheVerb);

            return verbs;
        }

        #endregion

        private class VerbContainer
        {
            ConfigurationConstsWriterNode _constWriterNode;
            DesignerVerb _verb;

            public VerbContainer(ConfigurationConstsWriterNode constWriterNode)
            {
                _constWriterNode = constWriterNode;
                _verb = new DesignerVerb(Resources.VerbRegenerate, new EventHandler(this.DoExecute));
            }

            public void DoExecute(object sender, EventArgs args)
            {
                string configurationDirectory = AppDomain.CurrentDomain.BaseDirectory;
                if (_constWriterNode.Hierarchy.RootNode.ConfigurationFile != null
                    && _constWriterNode.Hierarchy.RootNode.ConfigurationFile.Length != 0)
                {
                    configurationDirectory = Path.GetDirectoryName(_constWriterNode.Hierarchy.RootNode.ConfigurationFile);
                }

                ConfigurationConstsWriterSettings settings = ConfigurationConstsWriterSettingsBuilder.Build(_constWriterNode.Site, _constWriterNode);
                if (ConfigurationConstsWriter.Write(_constWriterNode, settings, configurationDirectory))
                {
                    Process notepadLauncher = new Process();
                    notepadLauncher.StartInfo.FileName = "notepad.exe";
                    notepadLauncher.StartInfo.Arguments = Path.Combine(configurationDirectory, settings.FilePath);
                    
                    //hell, why not do this async? it once hung in my IDE, you know.....
                    new Thread( delegate(){ try{ notepadLauncher.Start();} catch(Exception){}}).Start();
                }
            }


            public DesignerVerb TheVerb
            {
                get
                {
                    return _verb;
                }
            }

        }
    }
}
