using System;
using System.Collections.Generic;
using System.Text;
using System.ComponentModel;
using Microsoft.Practices.EnterpriseLibrary.Configuration.Design;
using EnterpriseLibraryExtensions.EnvironmentalOverrides.Properties;
using Microsoft.Practices.EnterpriseLibrary.Configuration.Design.Validation;

namespace EnterpriseLibraryExtensions.EnvironmentalOverrides
{

    [ProvideProperty(OverridesPropertyName, typeof(ConfigurationNode))]
    [Image(typeof(EnvironmentNode))]
    [SelectedImage(typeof(EnvironmentNode))]
    public class EnvironmentNode : ConfigurationNode, IExtenderProvider, ISitedConfigurationNodeSink
    {
        public const string OverridesPropertyName = "Overrides";

        string _path;

        IExtenderStorageService _extenderStorageService;
        ICurrentSelectionService _currentSelectionService;
        EnvironmentalOverrideConverter _environmentalOverridesConverter = new EnvironmentalOverrideConverter();

        public EnvironmentNode()
        {
            Rename(Resources.EnvrionmentNode);
        }

        public EnvironmentNode(string name, string path)
        {
            Name = name;
            _path = path;
        }

        [Required]
        [SRCategory("General", typeof(Resources))]
        [SRDescription("ConfigPath", typeof(Resources))]
        public string ConfigPath
        {
            get { return _path; }
            set { _path = value; }
        }

        #region IExtenderProvider Members
            
        [SRDescription("DescriptionOverrides", typeof(Resources))]
        [SRCategory("CategoryOverrides", typeof(Resources))]
        public EnvironmentalOverride GetOverrides(ConfigurationNode extendee)
        {
            EnvironmentalOverride overrides = _extenderStorageService.GetExtendedPropertyValue<EnvironmentalOverride>(this, OverridesPropertyName, extendee);
            if (overrides == null)
            {
                overrides = EnvironmentalOverride.Default;
            }
            overrides.SetObjectToCopy(extendee);

            return overrides;

        }

        public void SetOverrides(ConfigurationNode extendee, EnvironmentalOverride value)
        {
            _extenderStorageService.SetExtendedPropertyValue<EnvironmentalOverride>(this, OverridesPropertyName, extendee, value.UseParentProperties ? null : value);
        }


        public bool CanExtend(object extendee)
        {
            ConfigurationNode node = extendee as ConfigurationNode;

            if (node == null) return false;
            if (node is EnvironmentNode ||
                node is ConfigurationApplicationNode ||
                node is EnvironmentalOverridesNode) return false;

            if (node.Hierarchy != this.Hierarchy)
                return false;

            if (_currentSelectionService.CurrentlySelectedNode != extendee) return false;

            return true;
        }

        #endregion

        #region ISitedConfigurationNodeSink Members

        public void Sited()
        {
            _extenderStorageService = (IExtenderStorageService)Site.GetService(typeof(IExtenderStorageService));
            _currentSelectionService = (ICurrentSelectionService)Site.GetService(typeof(ICurrentSelectionService));
        }

        public void UnSited()
        {
        }

        #endregion
    }
}
