using System;
using System.Collections.Generic;
using System.Text;
using System.ComponentModel;
using System.Runtime.Serialization.Formatters.Soap;
using System.IO;
using System.Reflection;
using System.Runtime.Serialization;
using System.Collections;
using System.Configuration;
using System.Runtime.Serialization.Formatters.Binary;

namespace EnterpriseLibraryExtensions.EnvironmentalOverrides
{
    class EnvironmentalOverrideConverter : TypeConverter
    {
        public override bool CanConvertFrom(ITypeDescriptorContext context, Type sourceType)
        {
            return true;
        }

        public override object ConvertFrom(ITypeDescriptorContext context, System.Globalization.CultureInfo culture, object value)
        {
            BinaryFormatter formatter = new BinaryFormatter();
            formatter.SurrogateSelector = new ReflectionSurogateSelector();
            using (MemoryStream ms = new MemoryStream(Convert.FromBase64String((String)value)))
            {
                return formatter.Deserialize(ms);
            }
        }

        public override bool CanConvertTo(ITypeDescriptorContext context, Type destinationType)
        {
            return true;
        }

        public override object ConvertTo(ITypeDescriptorContext context, System.Globalization.CultureInfo culture, object value, Type destinationType)
        {
            BinaryFormatter formatter = new BinaryFormatter();
            formatter.SurrogateSelector = new ReflectionSurogateSelector();
            using (MemoryStream ms = new MemoryStream())
            {
                formatter.Serialize(ms, value);
                return Convert.ToBase64String(ms.ToArray());
            }
        }
        #region class ReflectionSurogateSelector
        class ReflectionSurogateSelector : ISurrogateSelector
        {
            Hashtable _surrogatesByType = new Hashtable();
            ISurrogateSelector _nextSelector;

            public void ChainSelector(ISurrogateSelector selector)
            {
                _nextSelector = selector;
            }

            public ISerializationSurrogate GetSurrogate(Type type, StreamingContext context, out ISurrogateSelector selector)
            {
                if (_surrogatesByType.ContainsKey(type))
                {
                    selector = this;
                    return (ISerializationSurrogate)_surrogatesByType[type];
                }

                if (!type.IsSerializable)
                {
                    ISerializationSurrogate surrogate = new ReflectionSurogate(type);
                    _surrogatesByType[type] = surrogate;

                    selector = this;
                    return surrogate;
                }
                selector = _nextSelector;
                return null;
            }

            public ISurrogateSelector GetNextSelector()
            {
                return _nextSelector;
            }

            private sealed class ReflectionSurogate : ISerializationSurrogate
            {
                Type _t;
                FieldInfo[] _fs;
                PropertyInfo[] _ps;

                public ReflectionSurogate(Type t)
                {
                    _t = t;
                    _fs = _t.GetFields();
                    _ps = _t.GetProperties();
                }

                #region ISerializationSurrogate Members

                public void GetObjectData(object obj, SerializationInfo info, StreamingContext context)
                {
                    foreach (FieldInfo f in _fs)
                    {
                        info.AddValue(f.Name, f.GetValue(obj));
                    }

                    foreach (PropertyInfo p in _ps)
                    {
                        if (p.CanRead && p.CanWrite && p.GetIndexParameters().Length == 0)
                        {
                            info.AddValue(p.Name, p.GetValue(obj, new object[0]));
                        }
                    }
                }

                public object SetObjectData(object obj, SerializationInfo info, StreamingContext context, ISurrogateSelector selector)
                {
                    foreach (PropertyInfo pi in _ps)
                    {
                        object pValue = info.GetValue(pi.Name, pi.PropertyType);
                        if (pValue != null)
                        {
                            pi.SetValue(obj, pValue, new object[0]);
                        }
                    }

                    foreach (FieldInfo fi in _fs)
                    {
                        object fValue = info.GetValue(fi.Name, fi.FieldType);
                        if (fValue != null)
                        {
                            fi.SetValue(obj, fValue);
                        }
                    }

                    return obj;
                }

                #endregion
            }
        #endregion
        }
    }
}
