using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Practices.EnterpriseLibrary.Configuration.Design;

namespace EnterpriseLibraryExtensions.EnvironmentalOverrides
{
    class EnvironmentalOverridesNodeBuilder
    {
        IConfigurationUIHierarchy _hierarchy;
        IExtenderStorageService _extenderStorageService;

        public EnvironmentalOverridesNodeBuilder(IServiceProvider serviceProvider)
        {
            _extenderStorageService = (IExtenderStorageService) serviceProvider.GetService(typeof(IExtenderStorageService));
            _hierarchy = ServiceHelper.GetCurrentHierarchy(serviceProvider);
        }

        public EnvironmentalOverridesNode Build(EnvironmentalOverridesSettings settings)
        {
            EnvironmentalOverridesNode overridesNode = new EnvironmentalOverridesNode();

            foreach (EnvironmentalOverrideSettings singleOverride in settings.Overrides)
            {
                EnvironmentNode environmentNode = new EnvironmentNode(singleOverride.Name, singleOverride.Path);
                _extenderStorageService.Load<EnvironmentalOverride>(environmentNode, EnvironmentNode.OverridesPropertyName, _hierarchy, singleOverride.OverridesByPath, delegate(string s) { return (EnvironmentalOverride)new EnvironmentalOverrideConverter().ConvertFrom(s); });

                overridesNode.AddNode(environmentNode);
            }

            return overridesNode;
        }

        internal static EnvironmentalOverridesNode Build(IServiceProvider serviceProvider, EnvironmentalOverridesSettings settings)
        {
            return new EnvironmentalOverridesNodeBuilder(serviceProvider).Build(settings);
        }
    }
}
