using System;
using System.Collections.Generic;
using System.Text;
using System.Configuration;
using System.ComponentModel;
using Microsoft.Practices.EnterpriseLibrary.Common.Configuration;

namespace EnterpriseLibraryExtensions.EnvironmentalOverrides
{
    public class EnvironmentalOverridesSettings : ConfigurationSection
    {
        public const string SectionName = "enterpriseLibraryExtensions.EnvironmentalOverrides";

        const string NameProperty = "name";
        const string OverridesProperty = "overrides";

        [ConfigurationProperty(NameProperty)]
        public string Name
        {
            get { return (string) base[NameProperty]; }
            set { base[NameProperty] = value; }
        }

        [ConfigurationProperty(OverridesProperty)]
        public EnvironmentalOverrideSettingsCollection Overrides
        {
            get { return (EnvironmentalOverrideSettingsCollection) base[OverridesProperty]; }
            set { base[OverridesProperty] = value; }
        }
    }

    [ConfigurationCollection(typeof(EnvironmentalOverrideSettings))]
    public class EnvironmentalOverrideSettingsCollection : ConfigurationElementCollection
    {
        protected override ConfigurationElement CreateNewElement()
        {
            return new EnvironmentalOverrideSettings();
        }

        protected override object GetElementKey(ConfigurationElement element)
        {
            return ((EnvironmentalOverrideSettings)element).Name;
        }

        public void Add(EnvironmentalOverrideSettings environment)
        {
            BaseAdd(environment);
        }
    }

    public class EnvironmentalOverrideSettings : ConfigurationElement
    {
        const string PathProperty = "path";
        const string NameProperty = "name";
        const string OverridesByPathProperty = "overridesByPath";
        
        [ConfigurationProperty(NameProperty)]
        public string Name
        {
            get { return (string) base[NameProperty]; }
            set { base[NameProperty] = value; }
        }

        [ConfigurationProperty(OverridesByPathProperty)]
        public KeyValueConfigurationCollection OverridesByPath
        {
            get { return (KeyValueConfigurationCollection)base[OverridesByPathProperty]; }
            set { base[OverridesByPathProperty] = value; }
        }

        [ConfigurationProperty(PathProperty)]
        public string Path
        {
            get { return (string)base[PathProperty]; }
            set { base[PathProperty] = value; }
        }
    }

    [ConfigurationCollection(typeof(OverrideSettings))]
    public class OverrideSettingsCollection : ConfigurationElementCollection
    {
        protected override ConfigurationElement CreateNewElement()
        {
            return new OverrideSettings();
        }

        protected override object GetElementKey(ConfigurationElement element)
        {
            OverrideSettings overrideSettings = element as OverrideSettings;
            if (overrideSettings != null)
            {
                return base.BaseGet(overrideSettings.Key);
            }
            return null;
        }
    }

    public class OverrideSettings : ConfigurationElement
    {
        private const string KeyProperty = "key";

        [ConfigurationProperty(KeyProperty, IsKey=true)]
        public string Key
        {
            get { return (string) base[KeyProperty]; }
            set { base[KeyProperty] = value; }
        }
    }
}
