using System;
using System.Collections.Generic;
using System.Text;
using System.Configuration;
using System.ComponentModel;

namespace EnterpriseLibraryExtensions.EnvironmentalOverrides
{
    class EnvironmentalOverridesSettingsBuilder
    {
        IExtenderStorageService _extenderStorageService;
        public EnvironmentalOverridesSettingsBuilder(IServiceProvider serviceProvider)
        {
            _extenderStorageService = (IExtenderStorageService) serviceProvider.GetService(typeof(IExtenderStorageService));
        }

        public EnvironmentalOverridesSettings Build(EnvironmentalOverridesNode node)
        {
            EnvironmentalOverridesSettings settings = new EnvironmentalOverridesSettings();
            settings.Name = node.Name;
            foreach (EnvironmentNode environment in node.Hierarchy.FindNodesByType(typeof(EnvironmentNode)))
            {
                EnvironmentalOverrideSettings environmentSettings = new EnvironmentalOverrideSettings();
                environmentSettings.Name = environment.Name;
                environmentSettings.Path = environment.ConfigPath;
                environmentSettings.OverridesByPath = _extenderStorageService.Save<EnvironmentalOverride>(environment, EnvironmentNode.OverridesPropertyName, delegate(EnvironmentalOverride v) { return (string) new EnvironmentalOverrideConverter().ConvertTo(v, typeof(string)); });

                settings.Overrides.Add(environmentSettings);
            }
            return settings;
        }

        internal static EnvironmentalOverridesSettings Build(IServiceProvider serviceProvider, EnvironmentalOverridesNode node)
        {
            return new EnvironmentalOverridesSettingsBuilder(serviceProvider).Build(node);
        }
    }
}
