using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing.Design;
using System.ComponentModel;

namespace EnterpriseLibraryExtensions.EnvironmentalOverrides
{
    public class WrappedEditor : UITypeEditor
    {
        UITypeEditor _actualEditor;

        public WrappedEditor(UITypeEditor actualEditor)
        {
            _actualEditor = actualEditor;
        }
        public override UITypeEditorEditStyle GetEditStyle(ITypeDescriptorContext context)
        {
            try
            {
                return _actualEditor.GetEditStyle(context);
            }
            catch (InvalidCastException)
            {
                EnvironmentalOverride env = context.Instance as EnvironmentalOverride;
                if (env != null)
                {
                    return _actualEditor.GetEditStyle(new WrappedContext(context, env.ParentNode));
                }

                return UITypeEditorEditStyle.None;
            }
        }

        public override bool GetPaintValueSupported(ITypeDescriptorContext context)
        {
            try
            {
                return _actualEditor.GetPaintValueSupported(context);
            }
            catch (InvalidCastException)
            {
                EnvironmentalOverride env = context.Instance as EnvironmentalOverride;
                if (env != null)
                {
                    return _actualEditor.GetPaintValueSupported(new WrappedContext(context, env.ParentNode));
                }

                return false;
            }
        }

        public override bool IsDropDownResizable
        {
            get
            {
                return _actualEditor.IsDropDownResizable;
            }
        }

        public override void PaintValue(PaintValueEventArgs e)
        {
            _actualEditor.PaintValue(e);
        }

        public override object EditValue(System.ComponentModel.ITypeDescriptorContext context, IServiceProvider provider, object value)
        {
            try
            {
                return _actualEditor.EditValue(context, provider, value);
            }
            catch(InvalidCastException)
            {
                EnvironmentalOverride env = context.Instance as EnvironmentalOverride;
                if (env != null)
                {
                    return _actualEditor.EditValue(new WrappedContext(context, env.ParentNode), provider, value);
                }
                return null;
            }
        }

        private class WrappedContext : ITypeDescriptorContext
        {
            ITypeDescriptorContext _actual;
            object _replacedInstance;

            public WrappedContext(ITypeDescriptorContext actual, object replacedInstance)
            {
                _actual = actual;
                _replacedInstance = replacedInstance;
            }

            #region ITypeDescriptorContext Members

            public IContainer Container
            {
                get { return _actual.Container ; }
            }

            public object Instance
            {
                get { return _replacedInstance; }
            }

            public void OnComponentChanged()
            {
                _actual.OnComponentChanged();
            }

            public bool OnComponentChanging()
            {
                return _actual.OnComponentChanging();
            }

            public PropertyDescriptor PropertyDescriptor
            {
                get { return _actual.PropertyDescriptor; }
            }

            #endregion

            #region IServiceProvider Members

            public object GetService(Type serviceType)
            {
                return _actual.GetService(serviceType);
            }

            #endregion
        }
    }
}
