using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Practices.EnterpriseLibrary.Configuration.Design;

namespace EnterpriseLibraryExtensions.NakedNode
{
    public class AddNakedNodeCommand : AddChildNodeCommand
    {
        private ConfigurationNode _childNode;
        private Type _childType;
        private object _initialValue;

        // Methods
        public AddNakedNodeCommand(IServiceProvider serviceProvider, Type childType, object initialValue)
            : base(serviceProvider, childType)
        {
            this._initialValue = initialValue;
            this._childType = childType;
        }

        protected override void ExecuteCore(ConfigurationNode node)
        {
            try
            {
                base.UIService.BeginUpdate();
                if (this._initialValue != null)
                {
                    try
                    {
                        this._childNode = Activator.CreateInstance(this._childType, new object[] { this._initialValue }) as ConfigurationNode;
                    }
                    catch
                    {
                    }
                }
                if (this._childNode == null)
                {
                    this._childNode = Activator.CreateInstance(this._childType) as ConfigurationNode;
                }
                node.AddNode(this._childNode);
                base.UIService.SetUIDirty(node.Hierarchy);
                base.UIService.ActivateNode(this._childNode);
            }
            finally
            {
                base.UIService.EndUpdate();
            }
        }

        public ConfigurationNode NakedChildNode
        {
            get
            {
                return this._childNode;
            }
        }
    }

 

}
