using System;
using System.Collections.Generic;
using System.Text;
using System.ComponentModel.Design;
using Microsoft.Practices.EnterpriseLibrary.Configuration.Design;
using EnterpriseLibraryExtensions;
using EnterpriseLibraryExtensions.NakedNode;
using System.Collections;
using System.Configuration;
using System.Reflection;

[assembly: ConfigDesignVerbExportAttribute(typeof(AddNewCollectionItemVerb))]
namespace EnterpriseLibraryExtensions.NakedNode
{
    public class AddNewCollectionItemVerb : IConfigDesignerVerb
    {
        public List<DesignerVerb> VerbsForCofigurationNode(ConfigurationNode node)
        {
            List<DesignerVerb> verbs = new List<DesignerVerb>();
            if (node is NakedConfigurationNode)
            {
                NakedConfigurationNode nakedNode = (NakedConfigurationNode) node;
                if (nakedNode is INakedCollectionNode)
                {
                    verbs.Add(new AddCollectionItemVerbContainer((IUIService)node.Site.GetService(typeof(IUIService)), (INakedCollectionNode)nakedNode).TheVerb);

                    foreach (object item in ((INakedCollectionNode)nakedNode).Collection)
                    {
                        verbs.Add(new RemoveCollectionItemVerbContainer((IUIService)node.Site.GetService(typeof(IUIService)), (INakedCollectionNode)nakedNode, item).TheVerb);
                    }
                }
            }
            return verbs;
        }

        private class RemoveCollectionItemVerbContainer
        {
            IUIService _uiService;
            INakedCollectionNode _collection;
            object _collectionItem;
            DesignerVerb _verb;

            public RemoveCollectionItemVerbContainer(IUIService uiService, INakedCollectionNode collection, object element)
            {
                _collectionItem = element;
                _uiService = uiService;
                _collection = collection;
                _verb = new DesignerVerb(string.Format("Remove {0}", collection.GetCollectionItemHeader(collection.Collection, element)), new EventHandler(DoExecute));
            }

            public DesignerVerb TheVerb
            {
                get { return _verb; }
            }

            private void DoExecute(object sender, EventArgs e)
            {
                _collection.RemoveCollectionItem(_collectionItem) ;

                _uiService.RefreshPropertyGrid();
            }
        }

        private class AddCollectionItemVerbContainer
        {
            IUIService _uiService;
            INakedCollectionNode _collection;
            DesignerVerb _verb;

            public AddCollectionItemVerbContainer(IUIService uiService, INakedCollectionNode collection)
            {
                _uiService = uiService;
                _collection = collection;
                _verb = new DesignerVerb(string.Format("Add new {0}", "Item"), new EventHandler(DoExecute));
            }

            public DesignerVerb TheVerb
            {
                get { return _verb; }
            }

            private void DoExecute(object sender, EventArgs e)
            {
                _collection.AddCollectionItem();

                _uiService.RefreshPropertyGrid();
            }
        }
    }
}
