using System;
using System.Collections.Generic;
using System.Text;
using System.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Configuration.Design;
using Microsoft.Practices.EnterpriseLibrary.Common.Configuration;

namespace EnterpriseLibraryExtensions.NakedNode
{
    public class AppSettingsDesignManager : NakedNodeDesignManager
    {
        public override void Register(IServiceProvider serviceProvider)
        {
            base.Register(serviceProvider);
            new AppSettingsCommandRegistrar(serviceProvider).Register();
        }

        protected override void OpenCore(IServiceProvider serviceProvider, ConfigurationApplicationNode rootNode, ConfigurationSection section)
        {
            if (section.ElementInformation.Source == null) return;

            base.OpenCore(serviceProvider, rootNode, section);
            if (((AppSettingsSection)section).Settings != null)
            {
                KeyValueCollectionNode keyValueNode = new KeyValueCollectionNode((AppSettingsSection)section);
                keyValueNode.Name = "Application Settings";
                rootNode.AddNode(keyValueNode);
            }
        }

        public override void Save(IServiceProvider serviceProvider)
        {
            ExeConfigurationFileMap fileMap = new ExeConfigurationFileMap();
            fileMap.ExeConfigFilename = ServiceHelper.GetApplicationConfigurationFile(serviceProvider);
            Configuration config = ConfigurationManager.OpenMappedExeConfiguration(fileMap, ConfigurationUserLevel.None);

            AppSettingsSection appsettingsSection = config.GetSection("appSettings") as AppSettingsSection;

            ConfigurationNode rootNode = ServiceHelper.GetCurrentRootNode(serviceProvider);
            KeyValueCollectionNode appSettingsNode = rootNode.Hierarchy.FindNodeByType(typeof(KeyValueCollectionNode)) as KeyValueCollectionNode;

            if (appsettingsSection != null && appSettingsNode != null)
            {
                appsettingsSection.File = appSettingsNode.TheObject.File;
                appSettingsNode.CopyTo(appsettingsSection);
            }
            else if (appSettingsNode == null)
            {
                appsettingsSection.SectionInformation.RevertToParent();
            }
            config.Save(ConfigurationSaveMode.Minimal);
        }

        protected override ConfigurationSectionInfo GetConfigurationSectionInfo(IServiceProvider serviceProvider)
        {
            return new ConfigurationSectionInfo(null, null, "appSettings");
        }


        private class AppSettingsCommandRegistrar : NakedNodeCommandRegistrar
        {
            public AppSettingsCommandRegistrar(IServiceProvider serviceProvider)
                : base(serviceProvider)
            {
            }

            public override void Register()
            {
                base.Register();

                AddNakedNodeCommand("Application Settings", typeof(AppSettingsSection), typeof(ConfigurationApplicationNode), NodeMultiplicity.Disallow);
            }
        }
    }
}
