using System;
using System.Collections.Generic;
using System.Text;
using System.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Configuration.Design;
using System.ComponentModel;
using EnterpriseLibraryExtensions.EnvironmentalOverrides;
using Microsoft.Practices.EnterpriseLibrary.Configuration.Design.Validation;
using System.Collections;

namespace EnterpriseLibraryExtensions.NakedNode
{

    [Image(typeof(KeyValueCollectionNode))]
    [SelectedImage(typeof(KeyValueCollectionNode))]
    //[Protectable("appSettings")]
    public class KeyValueCollectionNode : NakedConfigurationNodeCollection<AppSettingsSection, KeyValueCollectionNode.RWKeyValueConfigurationElement>
    {
        public KeyValueCollectionNode()
            : this(new AppSettingsSection())
        {
        }

        public KeyValueCollectionNode(AppSettingsSection appSettings)
            : base(appSettings, appSettings.Settings.EmitClear)
        {
            foreach (KeyValueConfigurationElement keyValueElement in appSettings.Settings)
            {
                AddCollectionItem(new RWKeyValueConfigurationElement(keyValueElement));
            }
        }

        protected override void InitializeCommands(NakedNodeCommandRegistrar commandRegistrar)
        {
        }

        public override void CopyTo(object target)
        {
            AppSettingsSection appSettingsSection = target as AppSettingsSection;
            if (appSettingsSection != null)
            {
                appSettingsSection.Settings.Clear();
                foreach (RWKeyValueConfigurationElement keyValue in Collection)
                {
                    appSettingsSection.Settings.Add(keyValue.CreateElement());
                }
                appSettingsSection.Settings.EmitClear = EmitClear;
            }
        }

        protected override void InitializeProperties(IPropertyAttributeService propertyAttributeService)
        {
            base.InitializeProperties(propertyAttributeService);

            propertyAttributeService.AddAttributeForProperty(typeof(KeyValueConfigurationCollection), "AllKeys", new BrowsableAttribute(false));
        }

        public class RWKeyValueConfigurationElement : CollectionItem<KeyValueConfigurationElement>
        {
            public RWKeyValueConfigurationElement()
            {
            }

            public RWKeyValueConfigurationElement(KeyValueConfigurationElement element)
                : base(element)
            {
            }

            public override string GetCategoryCaption()
            {
                return (string)this["key"];
            }

            public override KeyValueConfigurationElement CreateElement()
            {
                return new KeyValueConfigurationElement((string)base["key"], (string)base["value"]);
            }

            //#region IOverriddenPropertyNameProvider Members

            //public override string GetOverriddenPropertyName(string propertyName)
            //{
            //    if (string.Compare(propertyName, "Value") == 0)
            //    {
            //        return string.Format("{0}. {1}", Index, base["key"]);
            //    }
            //    return propertyName;
            //}

            //#endregion
        }
    }
}
