using System;
using System.Collections.Generic;
using System.Text;
using System.ComponentModel;

namespace EnterpriseLibraryExtensions.NakedNode
{
    class TimeSpanTypeConverter : TypeConverter
    {
        public override bool GetStandardValuesSupported(ITypeDescriptorContext context)
        {
            return true;
        }
        public override StandardValuesCollection GetStandardValues(ITypeDescriptorContext context)
        {
            List<TimeSpan> stdValues = new List<TimeSpan>();
            stdValues.Add(TimeSpan.Zero);
            stdValues.Add(new TimeSpan(0, 0, 1));
            stdValues.Add(new TimeSpan(0, 1, 0));
            stdValues.Add(new TimeSpan(0, 5, 0));
            stdValues.Add(new TimeSpan(0, 10, 0));
            stdValues.Add(new TimeSpan(1, 0, 0));
            stdValues.Add(TimeSpan.MaxValue);

            return new StandardValuesCollection(stdValues.ToArray());
        }

        public override bool GetStandardValuesExclusive(ITypeDescriptorContext context)
        {
            return false;
        }

        public override bool CanConvertFrom(ITypeDescriptorContext context, Type sourceType)
        {
            return (sourceType == typeof(string));
        }

        public override bool CanConvertTo(ITypeDescriptorContext context, Type destinationType)
        {
            return (destinationType == typeof(TimeSpan));
        }

        public override object ConvertFrom(ITypeDescriptorContext context, System.Globalization.CultureInfo culture, object value)
        {
            string a = (string)value;
            if (a == "Infinitive") return TimeSpan.MaxValue;

            TimeSpan t = TimeSpan.Zero;
            TimeSpan.TryParse(a, out t);

            return t;
        }

        public override object ConvertTo(ITypeDescriptorContext context, System.Globalization.CultureInfo culture, object value, Type destinationType)
        {
            if (destinationType.IsAssignableFrom(value.GetType()))
            {
                return value;
            }
            if (typeof(TimeSpan).IsAssignableFrom(value.GetType()))
            {
                TimeSpan t = (TimeSpan)value;
                if (t == TimeSpan.MaxValue) return "Infinitive";
                return t.ToString();
            }

            return null;
        }
    }
}
