using System;
using System.Collections.Generic;
using System.Text;
using System.ComponentModel;

namespace EnterpriseLibraryExtensions.NakedNode.Converters
{
    public class TypeTypeConverter : TypeConverter
    {
        public override bool CanConvertFrom(ITypeDescriptorContext context, Type sourceType)
        {
            return (typeof(string).IsAssignableFrom(sourceType) || typeof(Type).IsAssignableFrom(sourceType));
        }

        public override bool CanConvertTo(ITypeDescriptorContext context, Type destinationType)
        {
            return (typeof(Type).IsAssignableFrom(destinationType) || typeof(string).IsAssignableFrom(destinationType));
        }

        public override object ConvertFrom(ITypeDescriptorContext context, System.Globalization.CultureInfo culture, object value)
        {
            if (value is string)
            {
                string a = (string)value;
                if (a == null) return null;
                return Type.GetType(a);
            }
            else if (value is Type)
            {
                return ((Type)value).AssemblyQualifiedName;
            }

            return null;
        }

        public override object ConvertTo(ITypeDescriptorContext context, System.Globalization.CultureInfo culture, object value, Type destinationType)
        {
            if (destinationType == typeof(string))
            {
                if (value is string) return value;
                
                Type t = (Type)value;
                if (t == null) return string.Empty;
                return t.AssemblyQualifiedName;
            }
            else if (destinationType == typeof(Type))
            {
                if (value is Type) return value;

                string a = (string)value;
                if (a == null) return null;
                return Type.GetType(a);
            }

            return null;
        }
    }
}
