using System;
using System.Collections.Generic;
using System.Text;
using System.Reflection;
using Microsoft.Practices.EnterpriseLibrary.Configuration.Design;

namespace EnterpriseLibraryExtensions.NakedNode
{
    public class NakedNodeFactory : INakedNodeFactory
    {
        static Dictionary<Type, Type> NakedNodeTypeByDataType = new Dictionary<Type, Type>();

        static NakedNodeFactory()
        {
            foreach (Assembly loadedAssembly in AppDomain.CurrentDomain.GetAssemblies())
            {
                if (loadedAssembly.GetCustomAttributes(typeof(ConfigurationDesignManagerAttribute), true).Length != 0)
                {
                    foreach (Type t in loadedAssembly.GetTypes())
                    {
                        if (typeof(NakedConfigurationNode).IsAssignableFrom(t))
                        {
                            if (t.BaseType.IsGenericType)
                            {
                                NakedNodeTypeByDataType.Add(t.BaseType.GetGenericArguments()[0], t);
                            }
                        }
                    }
                }
            }
        }

        public NakedConfigurationNode CreateNakedConfigurationNode(Type dataType)
        {
            return (NakedConfigurationNode) Activator.CreateInstance(NodeTypeForDataType(dataType));
        }

        public Type NodeTypeForDataType(Type dataType)
        {
            if (NakedNodeTypeByDataType.ContainsKey(dataType))
            {
                return NakedNodeTypeByDataType[dataType];
            }
            else
            {
                if (dataType.GetConstructor(new Type[0]) != null)
                {
                    return typeof(NakedConfigurationNode<>).MakeGenericType(dataType);
                }
                else
                {
                    return typeof(NakedConfigurationNode);
                }
            }
        }

        public void SetNodeTypeForDataType(Type dataType, Type nodeType)
        {
            NakedNodeTypeByDataType[dataType] = nodeType;
        }
    }

    public interface INakedNodeFactory
    {
        NakedConfigurationNode CreateNakedConfigurationNode(Type dataType);
        Type NodeTypeForDataType(Type dataType);
        void SetNodeTypeForDataType(Type dataType, Type nodeType);
    }
}
