using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Practices.EnterpriseLibrary.Configuration.Design;
using System.Windows.Forms;
using System.Reflection;
using System.Configuration;

namespace EnterpriseLibraryExtensions.NakedNode
{
    public class AddNakedNodeConfigurationUICommand : ConfigurationUICommand
    {
        Type _childNodeType;
        public AddNakedNodeConfigurationUICommand(IServiceProvider serviceProvider, string name, string longName, CommandState state, NodeMultiplicity multiplicity, AddChildNodeCommand command, Type childNodeType)
            : base(serviceProvider, name, longName, state, multiplicity, command, childNodeType, Shortcut.None, InsertionPoint.New, null)
        {
            _childNodeType = childNodeType;
        }

        public Type ChildType
        {
            get { return _childNodeType; }
        }
    }

    public class NakedNodeCommandRegistrar : CommandRegistrar
    {
        INakedNodeFactory nodeFactory;
        INodeNameCreationService nodeNameService;
        List<Type> _disabledDataTypes = new List<Type>();
        List<Type> _nodeTypesWithDefaultCommands = new List<Type>();
        Dictionary<Type, List<Type>> _addCommandsByRegistrar = new Dictionary<Type, List<Type>>();
           
        public NakedNodeCommandRegistrar(IServiceProvider serviceProvider)
            :base(serviceProvider)
        {
            nodeFactory = (INakedNodeFactory)serviceProvider.GetService(typeof(INakedNodeFactory));
            nodeNameService = (INodeNameCreationService)serviceProvider.GetService(typeof(INodeNameCreationService));
        }

        public void AddNakedNodeCommand(string name, Type dataType, Type registerType, NodeMultiplicity nodeMultiplicity)
        {
            AddNakedNodeCommand(name, dataType, registerType, nodeMultiplicity, null);
        }
        public void AddNakedNodeCommand(string name, Type dataType, Type registerType, NodeMultiplicity nodeMultiplicity, object initialValue)
		{
            if (!_addCommandsByRegistrar.ContainsKey(registerType))
            {
                _addCommandsByRegistrar.Add(registerType, new List<Type>());
            }
            if (!_addCommandsByRegistrar[registerType].Contains(dataType))
            {
                Type nodeType = nodeFactory.NodeTypeForDataType(dataType);


                AddNakedNodeCommand addChildNodeCommand = new AddNakedNodeCommand(ServiceProvider, nodeType, initialValue);
                addChildNodeCommand.Executed += delegate(object sender, EventArgs args)
                {
                    ((AddNakedNodeCommand)sender).NakedChildNode.Name = nodeNameService.GetUniqueName(name, ((AddNakedNodeCommand)sender).NakedChildNode, ((AddNakedNodeCommand)sender).NakedChildNode.Parent);
                };

                ConfigurationUICommand item = new AddNakedNodeConfigurationUICommand(ServiceProvider, name,
                    string.Format("Add {0} node.", name),
                    CommandState.Enabled, nodeMultiplicity,
                    addChildNodeCommand,
                    nodeType);

                if (!_nodeTypesWithDefaultCommands.Contains(nodeType))
                {

                    AddValidateCommand(nodeType);
                    if (typeof(ConfigurationSection).IsAssignableFrom(dataType))
                    {
                        AddRemoveCommand(nodeType);
                    }

                    AddUICommand(item, registerType);
                    _nodeTypesWithDefaultCommands.Add(nodeType);

                }
                _addCommandsByRegistrar[registerType].Add(dataType);
            }
		}
        public void RegisterDefaultChildCommands(Type dataType)
        {
            RegisterDefaultChildCommands(dataType, null);
        }

        public void RegisterDefaultChildCommands(Type dataType, object o)
        {
            foreach (PropertyInfo property in dataType.GetProperties())
            {
                if (typeof(ConfigurationElement).IsAssignableFrom(property.PropertyType)
                    && property.PropertyType != typeof(ConfigurationSection)
                    && property.PropertyType != typeof(IgnoreSection)
                    && property.PropertyType != typeof(DefaultSection)
                    && !_disabledDataTypes.Contains(property.PropertyType))
                {
                    if (property.GetIndexParameters().Length == 0)
                    {
                        object propertyValue = null;
                        if (o != null)
                        {
                            propertyValue = property.GetValue(o, new object[0]);
                        }
                        AddNakedNodeCommand(property.Name, property.PropertyType, nodeFactory.NodeTypeForDataType(dataType), NodeMultiplicity.Disallow, propertyValue);
                    }
                }
            }
        }

        public void DisableResigtration(Type dataType)
        {
            if (!_disabledDataTypes.Contains(dataType))
            {
                _disabledDataTypes.Add(dataType);
            }
        }

        public override void Register()
        {
        }
    }
}
