using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Practices.EnterpriseLibrary.Configuration.Design;
using EnterpriseLibraryExtensions.NakedNode;
using System.ComponentModel.Design;
using EnterpriseLibraryExtensions;
using System.Configuration;

namespace EnterpriseLibraryExtensions.NakedNode
{
    public class NakedNodeDesignManager : ConfigurationDesignManager
    {
        public override void Register(IServiceProvider serviceProvider)
        {
            if (serviceProvider.GetService(typeof(IPropertyAttributeService)) == null)
            {
                IServiceContainer container = (IServiceContainer)serviceProvider.GetService(typeof(IServiceContainer));
                container.AddService(typeof(IPropertyAttributeService), new PropertyAttributeService());
                container.AddService(typeof(ITypeConverterService), new TypeConverterService());
                container.AddService(typeof(INakedNodeFactory), new NakedNodeFactory());
                container.AddService(typeof(ITypeEditorService), new TypeEditorService());
            }
        }

        protected override void OpenCore(IServiceProvider serviceProvider, ConfigurationApplicationNode rootNode, System.Configuration.ConfigurationSection section)
        {
            base.OpenCore(serviceProvider, rootNode, section);
        }

        protected void CreateClone(ConfigurationElement source, ConfigurationElement dest)
        {
            Dictionary<string, PropertyInformation> DestPropertiesByName = new Dictionary<string, PropertyInformation>();
            foreach (PropertyInformation ci in dest.ElementInformation.Properties)
            {
                DestPropertiesByName.Add(ci.Name, ci);
            }
            foreach (PropertyInformation ci in source.ElementInformation.Properties)
            {
                if (DestPropertiesByName.ContainsKey(ci.Name))
                {
                    DestPropertiesByName[ci.Name].Value = ci.Value;
                }
            }
        }
    }
}
