using System;
using System.Text;
using System.Configuration;
using System.Collections.Generic;
using Microsoft.Practices.EnterpriseLibrary.Configuration.Design;

namespace EnterpriseLibraryExtensions.Protection
{
    public class ProtectionConfigurationDesignManager :ConfigurationDesignManager
    {
        IConfigurationManagementService _configurationManagementService;
        IExtenderStorageService _extenderStorageService;

        public override void Register(IServiceProvider serviceProvider)
        {
            IGlobalEvents extendedEventService = serviceProvider.GetService(typeof(IGlobalEvents)) as IGlobalEvents;
            _extenderStorageService = serviceProvider.GetService(typeof(IExtenderStorageService)) as IExtenderStorageService;
            _configurationManagementService = serviceProvider.GetService(typeof(IConfigurationManagementService)) as IConfigurationManagementService;

            extendedEventService.HierarchySaved += new EventHandler<HierarchySavedEventArgs>(extendedEventService_HierarchySaved);

            ProtectionCommandRegistrar.Register(serviceProvider);
        }

        void extendedEventService_HierarchySaved(object sender, HierarchySavedEventArgs e)
        {
            foreach (ProtectionNode protectionNode in e.UIHierarchy.FindNodesByType(typeof(ProtectionNode)))
            {
                bool shouldSave = false;
                Configuration configuration = _configurationManagementService.GetConfiguration(e.UIHierarchy);
                KeyValueConfigurationCollection sectionsToProtect = _extenderStorageService.Save<string>(protectionNode, ProtectionNode.ProtectProperty, null);
                foreach (string key in sectionsToProtect.AllKeys)
                {
                    string configurationSectionName = sectionsToProtect[key].Value;
                    ConfigurationSection sectionToProtect = configuration.GetSection(configurationSectionName);
                    if (sectionToProtect != null && (!sectionToProtect.SectionInformation.IsProtected || sectionToProtect.SectionInformation.ProtectionProvider.Name != protectionNode.ProtectionProvider))
                    {
                        sectionToProtect.SectionInformation.ProtectSection(protectionNode.ProtectionProvider);
                        sectionToProtect.SectionInformation.ForceSave = true;
                        shouldSave = true;
                    }
                }

                if (shouldSave)
                {
                    configuration.Save(ConfigurationSaveMode.Full);
                    _configurationManagementService.FlushConfigurationSourceCache(e.UIHierarchy);
                }
            }
        }

        protected override ConfigurationSectionInfo GetConfigurationSectionInfo(IServiceProvider serviceProvider)
        {
            IConfigurationUIHierarchy hierarchy = ServiceHelper.GetCurrentHierarchy(serviceProvider);
            ProtectionNode node = (ProtectionNode) hierarchy.FindNodeByType(typeof(ProtectionNode));
            ProtectionSettings settings = null;
            if (node != null)
            {
                settings = ProtectionSettingsBuilder.Build(serviceProvider, node);
            }
            return new ConfigurationSectionInfo(node, settings, ProtectionSettings.SectionName);
        }


        protected override void OpenCore(IServiceProvider serviceProvider, ConfigurationApplicationNode rootNode, System.Configuration.ConfigurationSection section)
        {
            ProtectionSettings protectionSettings = section as ProtectionSettings;
            if (protectionSettings != null)
            {
                rootNode.AddNode(ProtectionNodeBuilder.Build(serviceProvider, rootNode.Hierarchy, protectionSettings));
            }
        }
    }
}
