using System;
using System.Collections.Generic;
using System.Text;
using EnterpriseLibraryExtensions.NakedNode;
using System.Web.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Configuration.Design;
using System.ComponentModel;
using Microsoft.Practices.EnterpriseLibrary.Configuration.Design.Validation;

namespace EnterpriseLibraryExtensions.SystemWeb
{
    [SelectedImage(typeof(BufferModesCollectionNode))]
    [Image(typeof(BufferModesCollectionNode))]
    public class BufferModesCollectionNode : NakedConfigurationNodeCollection<BufferModesCollection, BufferModesCollectionNode.RWBufferMode>
    {
        public BufferModesCollectionNode()
        {
        }

        public BufferModesCollectionNode(BufferModesCollection data)
            :base(data, data.EmitClear)
        {
            foreach (BufferModeSettings bm in data)
            {
                AddCollectionItem(new RWBufferMode(bm));
            }
        }

        public override void CopyTo(object target)
        {
            BufferModesCollection BufferModesCollection = target as BufferModesCollection;

            if (BufferModesCollection != null)
            {
                BufferModesCollection.Clear();
                foreach (RWBufferMode bufferMode in Collection)
                {
                    BufferModesCollection.Add(bufferMode.CreateElement());
                }
                BufferModesCollection.EmitClear = EmitClear;
            }
        }

        public class RWBufferMode : CollectionItem<BufferModeSettings>
        {
            public RWBufferMode()
            {
            }

            public RWBufferMode(BufferModeSettings bufferMode)
                :base(bufferMode)
            {
            }

            public override void Validate(IList<ValidationError> errors)
            {
                base.Validate(errors);
                if ((TimeSpan)base["regularFlushInterval"] == TimeSpan.MaxValue)
                {
                    errors.Add(new ValidationError(Owner, "Collection", string.Format("Property RegularFlushInterval on '{0}' should be less than Invinitive", this.GetCategoryCaption())));
                }

                if ((TimeSpan)base["urgentFlushInterval"] == TimeSpan.MaxValue)
                {
                    errors.Add(new ValidationError(Owner, "Collection", string.Format("Property UrgentFlushInterval on '{0}' should be less than Invinitive", this.GetCategoryCaption())));
                }

            }

            public override BufferModeSettings CreateElement()
            {
                return new BufferModeSettings((string)base["name"], (int)base["maxBufferSize"], (int)base["maxFlushSize"], (int)base["urgentFlushThreshold"], (TimeSpan)base["regularFlushInterval"], (TimeSpan)base["urgentFlushInterval"], (int)base["maxBufferThreads"]);
            }
        }
    }
}
