using System;
using System.Collections.Generic;
using System.Text;
using EnterpriseLibraryExtensions.NakedNode;
using System.Web.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Configuration.Design;
using System.ComponentModel;
using System.Drawing.Design;

namespace EnterpriseLibraryExtensions.SystemWeb
{
    [SelectedImage(typeof(BuildProviderCollectionNode))]
    [Image(typeof(BuildProviderCollectionNode))]
    public class BuildProviderCollectionNode : NakedConfigurationNodeCollection<BuildProviderCollection, BuildProviderCollectionNode.RWBuildProvider>
    {
        public BuildProviderCollectionNode()
        {
        }

        public BuildProviderCollectionNode(BuildProviderCollection data)
            : base(data, data.EmitClear)
        {
            foreach (BuildProvider bp in data)
            {
                AddCollectionItem(new RWBuildProvider(bp));
            }
        }

        public override void CopyTo(object target)
        {
            BuildProviderCollection buildProviders = target as BuildProviderCollection;

            if (buildProviders != null)
            {
                buildProviders.Clear();
                foreach (RWBuildProvider buildProvider in Collection)
                {
                    buildProviders.Add(buildProvider.CreateElement());
                }
                buildProviders.EmitClear = EmitClear;
            }
        }

        public override void AddCollectionItem()
        {
            using (TypeSelectorUI selectorUI = new TypeSelectorUI(typeof(BuildProvider), typeof(BuildProvider)))
            {
                if (selectorUI.ShowDialog() == System.Windows.Forms.DialogResult.OK)
                {
                    RWBuildProvider buildProvider = new RWBuildProvider(new BuildProvider("", selectorUI.SelectedType.AssemblyQualifiedName));
                    base.AddCollectionItem(buildProvider);
                }
            }
        }

        protected override void InitializeProperties(IPropertyAttributeService propertyAttributeService)
        {
            base.InitializeProperties(propertyAttributeService);

            propertyAttributeService.AddAttributeForProperty(typeof(BuildProvider), "Type", new ReadOnlyAttribute(true));
            propertyAttributeService.AddAttributeForProperty(typeof(BuildProvider), "Type", new EditorAttribute(typeof(StringTypeSelectorEditor), typeof(UITypeEditor)));
            propertyAttributeService.AddAttributeForProperty(typeof(BuildProvider), "Type", new BaseTypeAttribute(typeof(BuildProvider)));
        }

        public class RWBuildProvider : CollectionItem<BuildProvider>
        {

            public RWBuildProvider()
            {
            }

            public RWBuildProvider(BuildProvider buildProvider)
                : base(buildProvider)
            {
            }

            public override BuildProvider CreateElement()
            {
                return new BuildProvider((string)base["extension"], (string)base["type"]);
            }
        }
    }
}
