using System;
using System.Collections.Generic;
using System.Text;
using EnterpriseLibraryExtensions.NakedNode;
using System.Web.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Configuration.Design;
using System.ComponentModel;
using EnterpriseLibraryExtensions.EnvironmentalOverrides;
using Microsoft.Practices.EnterpriseLibrary.Configuration.Design.Validation;

namespace EnterpriseLibraryExtensions.SystemWeb
{
    [SelectedImage(typeof(CustomErrorsSectionNode))]
    [Image(typeof(CustomErrorsSectionNode))]
    public class CustomErrorsSectionNode : NakedConfigurationNodeCollection<CustomErrorsSection, CustomErrorsSectionNode.RWCustomError>
    {
        public CustomErrorsSectionNode()
            : this(new CustomErrorsSection())
        {
        }

        public CustomErrorsSectionNode(CustomErrorsSection data)
            :base(data, data.Errors.EmitClear, false)
        {

            foreach (CustomError error in data.Errors)
            {
                AddCollectionItem(new RWCustomError(error));
            }
        }

        public override void CopyTo(object target)
        {
            CustomErrorsSection customErrors = target as CustomErrorsSection;
            if (customErrors == null) return;

            customErrors.Errors.Clear();
            foreach (RWCustomError customError in Collection)
            {
                customErrors.Errors.Add(customError.CreateElement());
            }
            customErrors.Errors.EmitClear = EmitClear;
        }

        protected override void InitializeCommands(NakedNodeCommandRegistrar commandRegistrar)
        {
        }

        public class RWCustomError : CollectionItem<CustomError>
        {
            public RWCustomError()
            {
                base["statusCode"] = 404;
            }

            public RWCustomError(CustomError customError)
                : base(customError)
            {
            }

            //public override string GetOverriddenPropertyName(string propertyName)
            //{
            //    return string.Format("Redirect for statuscode {0}", base["statusCode"]);
            //}

            public override CustomError CreateElement()
            {
                return new CustomError(StatusCode, (string)base["redirect"]);
            }

            private int StatusCode
            {
                get { return (int)base["statusCode"]; }
            }

            public override string GetCategoryCaption()
            {
                return string.Format("Statuscode {0}", base["statusCode"]);
            }

            public override void Validate(IList<ValidationError> errors)
            {
                base.Validate(errors);
                if (StatusCode < 100 || StatusCode > 999)
                {
                    errors.Add(new ValidationError(this.Owner, "Collection", "Statuscode value should be between 100-999.")); 
                }
            }
        }
    }

    
}
