using System;
using System.Collections.Generic;
using System.Text;
using System.Web.Configuration;
using System.Web;
using EnterpriseLibraryExtensions.NakedNode;
using Microsoft.Practices.EnterpriseLibrary.Configuration.Design;
using System.Configuration;
using System.ComponentModel;
using Microsoft.Practices.EnterpriseLibrary.Configuration.Design.Validation;
using System.Drawing.Design;

namespace EnterpriseLibraryExtensions.SystemWeb
{
    [Image(typeof(HttpHandlersSectionNode))]
    [SelectedImage(typeof(HttpHandlersSectionNode))]
    public class HttpHandlersSectionNode : NakedConfigurationNodeCollection<HttpHandlersSection, RWHttpHandlerAction>
    {
        public HttpHandlersSectionNode()
            :this(new HttpHandlersSection())
        {
        }

        public HttpHandlersSectionNode(HttpHandlersSection httpHandlers)
            :base(httpHandlers, httpHandlers.Handlers.EmitClear)
        {
            foreach (HttpHandlerAction handler in httpHandlers.Handlers)
            {
                AddCollectionItem(new RWHttpHandlerAction(handler));
            }
        }
        public override void CopyTo(object target)
        {
            HttpHandlersSection handlersSection = target as HttpHandlersSection;
            if (handlersSection == null) return;

            handlersSection.Handlers.Clear();

            foreach (RWHttpHandlerAction handler in Collection)
            {
                handlersSection.Handlers.Add(handler.CreateElement());
            }
            handlersSection.Handlers.EmitClear = EmitClear;
        }

        public override void AddCollectionItem()
        {
            using (TypeSelectorUI selector = new TypeSelectorUI(typeof(IHttpHandler), typeof(IHttpHandler)))
            {
                if (selector.ShowDialog() == System.Windows.Forms.DialogResult.OK)
                {
                    RWHttpHandlerAction handlerAction = new RWHttpHandlerAction(selector.SelectedType);
                    base.AddCollectionItem(handlerAction);
                }
            }
        }

        protected override void InitializeCommands(NakedNodeCommandRegistrar commandRegistrar)
        {
            
        }

        protected override void InitializeProperties(IPropertyAttributeService propertyAttributeService)
        {
            propertyAttributeService.AddAttributeForProperty(typeof(HttpHandlerAction), "Type", new EditorAttribute(typeof(StringTypeSelectorEditor), typeof(UITypeEditor)));
            propertyAttributeService.AddAttributeForProperty(typeof(HttpHandlerAction), "Type", new BaseTypeAttribute(typeof(IHttpHandler)));

            base.InitializeProperties(propertyAttributeService);
        }
    }

    public class RWHttpHandlerAction : CollectionItem<HttpHandlerAction>
    {
        public RWHttpHandlerAction()
        {
        }

        public RWHttpHandlerAction(Type t)
        {
            base["type"] = t.AssemblyQualifiedName;
        }

        public RWHttpHandlerAction(HttpHandlerAction handler)
            :base(handler)
        {
        }

        public override HttpHandlerAction CreateElement()
        {
            return new HttpHandlerAction((string)base["path"], (string)base["type"], (string)base["verb"]);
        }
    }
}
