using System;
using System.Collections.Generic;
using System.Text;
using EnterpriseLibraryExtensions.NakedNode;
using System.Web.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Configuration.Design;
using System.Web;
using System.ComponentModel;
using System.Configuration;
using System.Drawing.Design;
using Microsoft.Practices.EnterpriseLibrary.Configuration.Design.Validation;


namespace EnterpriseLibraryExtensions.SystemWeb
{
    [Image(typeof(HttpModulesSectionNode))]
    [SelectedImage(typeof(HttpModulesSectionNode))]
    public class HttpModulesSectionNode : NakedConfigurationNodeCollection<HttpModulesSection, RWHttpModuleAction>
    {
        public HttpModulesSectionNode()
            :this(new HttpModulesSection())
        {
        }

        public HttpModulesSectionNode(HttpModulesSection httpModules)
            :base(httpModules, httpModules.Modules.EmitClear)
        {
            foreach (HttpModuleAction module in httpModules.Modules)
            {
                AddCollectionItem(new RWHttpModuleAction( module) );
            }
        }

        protected override void InitializeCommands(NakedNodeCommandRegistrar commandRegistrar)
        {
            
        }

        protected override void InitializeProperties(IPropertyAttributeService propertyAttributeService)
        {
            propertyAttributeService.AddAttributeForProperty(typeof(HttpModuleAction), "Type", new EditorAttribute(typeof(StringTypeSelectorEditor), typeof(UITypeEditor)));
            propertyAttributeService.AddAttributeForProperty(typeof(HttpModuleAction), "Type", new BaseTypeAttribute(typeof(IHttpModule)));

            base.InitializeProperties(propertyAttributeService);
        }
        
        public override void CopyTo(object o)
        {
            HttpModulesSection modulesSection = o as HttpModulesSection;
            if (modulesSection == null) return;

            modulesSection.Modules.Clear();

            foreach (RWHttpModuleAction module in Collection)
            {
                modulesSection.Modules.Add(module.CreateElement());
            }
            modulesSection.Modules.EmitClear = EmitClear;
        }

        public override void AddCollectionItem()
        {
            using (TypeSelectorUI selector = new TypeSelectorUI(typeof(IHttpModule), typeof(IHttpModule)))
            {
                if (selector.ShowDialog() == System.Windows.Forms.DialogResult.OK)
                {
                    RWHttpModuleAction moduleAction = new RWHttpModuleAction(selector.SelectedType.Name, selector.SelectedType);
                    base.AddCollectionItem(moduleAction);
                }
            }
        }
    }

    public class RWHttpModuleAction : CollectionItem<HttpModuleAction>
    {
        public RWHttpModuleAction()
        {
        }

        public RWHttpModuleAction(HttpModuleAction moduleAction)
            :base(moduleAction)
        {
        }

        public RWHttpModuleAction(string name, Type t)
        {
            base["name"] = name;
            base["type"] = t.AssemblyQualifiedName;

        }

        public override HttpModuleAction CreateElement()
        {
            return new HttpModuleAction((string)base["name"], (string)base["type"]); 
        }
    }
}
