using System;
using System.Collections.Generic;
using System.Text;
using EnterpriseLibraryExtensions.NakedNode;
using System.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Configuration.Design;
using System.Collections.Specialized;
using System.Collections;
using System.ComponentModel;
using Microsoft.Practices.EnterpriseLibrary.Configuration.Design.Validation;
using System.Drawing.Design;
using System.Configuration.Provider;
using EnterpriseLibraryExtensions.NakedNode.Converters;

namespace EnterpriseLibraryExtensions.SystemWeb
{

    [SelectedImage(typeof(ProviderSettingsCollectionNode))]
    [Image(typeof(ProviderSettingsCollectionNode))]
    public class ProviderSettingsCollectionNode : NakedConfigurationNodeCollection<ProviderSettingsCollection, RWProviderSettings>
    {

        public ProviderSettingsCollectionNode(ProviderSettingsCollection data)
            : base(data, data.EmitClear)
        {
            foreach (ProviderSettings provider in data)
            {
                AddCollectionItem(new RWProviderSettings(provider));
            }
        }

        public ProviderSettingsCollectionNode()
            :this(new ProviderSettingsCollection())
        {
        }

        public override void AddCollectionItem()
        {
            using (TypeSelectorUI selector = new TypeSelectorUI(typeof(ProviderBase), typeof(ProviderBase)))
            {
                if (selector.ShowDialog() == System.Windows.Forms.DialogResult.OK)
                {
                    RWProviderSettings provider = new RWProviderSettings(selector.SelectedType.Name, selector.SelectedType);
                    base.AddCollectionItem(provider);
                }
            }
        }

        public override void CopyTo(object target)
        {
            ProviderSettingsCollection providerSettingsCollection = target as ProviderSettingsCollection;
            if (providerSettingsCollection == null) return;

            providerSettingsCollection.Clear();
            
            foreach (RWProviderSettings provider in Collection)
            {
                providerSettingsCollection.Add(provider.CreateElement());
            }

            providerSettingsCollection.EmitClear = EmitClear;
        }

        protected override void InitializeProperties(IPropertyAttributeService propertyAttributeService)
        {

            propertyAttributeService.AddAttributeForProperty(typeof(ProviderSettings), "Type", new TypeAttribute(typeof(Type), typeof(TypeTypeConverter)));
            propertyAttributeService.AddAttributeForProperty(typeof(ProviderSettings), "Type", new EditorAttribute(typeof(TypeSelectorEditor), typeof(UITypeEditor)));
            propertyAttributeService.AddAttributeForProperty(typeof(ProviderSettings), "Type", new BaseTypeAttribute(typeof(ProviderBase)));

            base.InitializeProperties(propertyAttributeService);
        }
    }

    public class RWProviderSettings : CollectionItem<ProviderSettings>
    {
        Dictionary<string, object> _attributes = new Dictionary<string,object>();

        public RWProviderSettings()
        {
        }

        public RWProviderSettings(string name, Type t)
        {
            base["name"] = name;
            base["type"] = t.AssemblyQualifiedName;
        }

        public RWProviderSettings(ProviderSettings providerSettings)
            :base(providerSettings)
        {
        }

        public override ProviderSettings CreateElement()
        {
            ProviderSettings providerSettings = new ProviderSettings((string)base["name"], (string)base["type"]);
            //TODO
            //providerSettings.Parameters = (NameValueCollection) base["parameters"];

            return providerSettings;
        }

        public string Name
        {
            get { return (string)base["name"]; }
        }

        protected override object UnkownPropertyGet(string name)
        {
            if (_attributes.ContainsKey(name))
            {
                return _attributes[name];
            }
            return null;
        }

        protected override void UnkownPropertySet(string name, object value)
        {
            _attributes[name] = value;
        }
    }
}
