using System;
using System.Collections.Generic;
using System.Text;
using EnterpriseLibraryExtensions.NakedNode;
using System.Web.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Configuration.Design;
using System.ComponentModel;

namespace EnterpriseLibraryExtensions.SystemWeb
{
    [SelectedImage(typeof(RootProfilePropertySettingsCollectionNode))]
    [Image(typeof(RootProfilePropertySettingsCollectionNode))]
    public class RootProfilePropertySettingsCollectionNode : NakedConfigurationNodeCollection<RootProfilePropertySettingsCollection, RootProfilePropertySettingsCollectionNode.RWProfileSettings>, ISitedConfigurationNodeSink
    {
        public RootProfilePropertySettingsCollectionNode()
        {
        }

        public RootProfilePropertySettingsCollectionNode(RootProfilePropertySettingsCollection data)
            : base(data, data.EmitClear)
        {
            foreach (ProfilePropertySettings profileSetting in data)
            {
                AddCollectionItem(new RWProfileSettings(profileSetting));
                
            }
            
        }

        public override void Validate(IList<Microsoft.Practices.EnterpriseLibrary.Configuration.Design.Validation.ValidationError> errors)
        {
            //TODO
            base.Validate(errors);
        }

        public override void CopyTo(object target)
        {
            RootProfilePropertySettingsCollection profileSettings = target as RootProfilePropertySettingsCollection;
            if (profileSettings != null)
            {
                profileSettings.Clear();
                foreach (RWProfileSettings profileSetting in Collection)
                {
                    profileSettings.Add(profileSetting.CreateElement());
                }

                profileSettings.GroupSettings.Clear();
                foreach(ProfilePropertySettingsCollectionNode propertyCollectionNode in Nodes)
                {
                    ProfileGroupSettings groupSettings = new ProfileGroupSettings(propertyCollectionNode.Name);
                    propertyCollectionNode.CopyTo(groupSettings.PropertySettings);

                    profileSettings.GroupSettings.Add(groupSettings);
                }
                profileSettings.EmitClear = EmitClear;
            }
        }
        public override void Sited()
        {
            base.Sited();
            foreach (ProfileGroupSettings profileGroup in TheObject.GroupSettings)
            {
                ConfigurationNode childNode = new ProfilePropertySettingsCollectionNode(profileGroup.PropertySettings);
                childNode.Name = profileGroup.Name;
                AddNode(childNode);
            }
        }


        protected override void InitializeCommands(NakedNodeCommandRegistrar commandRegistrar)
        {
            commandRegistrar.AddNakedNodeCommand("PropertyGroup", typeof(ProfilePropertySettingsCollection), typeof(RootProfilePropertySettingsCollectionNode), NodeMultiplicity.Allow);
        }

        public class RWProfileSettings : CollectionItem<ProfilePropertySettings>
        {
            public RWProfileSettings()
            {
            }

            public RWProfileSettings(ProfilePropertySettings profileSettings)
                : base(profileSettings)
            {
            }

            public override ProfilePropertySettings CreateElement()
            {
                return new ProfilePropertySettings((string)base["name"], (bool)base["readOnly"], (SerializationMode)base["serializeAs"], (string)base["provider"], (string)base["defaultValue"], (string)base["type"], (bool)base["allowAnonymous"], (string)base["customProviderData"]);
            }
        }
    }
}
