using System;
using System.Collections.Generic;
using System.Text;
using System.Web.Configuration;
using EnterpriseLibraryExtensions.NakedNode;
using System.ComponentModel;
using Microsoft.Practices.EnterpriseLibrary.Configuration.Design.Validation;
using Microsoft.Practices.EnterpriseLibrary.Configuration.Design;

namespace EnterpriseLibraryExtensions.SystemWeb
{
    [SelectedImage(typeof(UrlMappingsSectionNode))]
    [Image(typeof(UrlMappingsSectionNode))]
    public class UrlMappingsSectionNode : NakedConfigurationNodeCollection<UrlMappingsSection, UrlMappingsSectionNode.RWUrlMapping>
    {
        public UrlMappingsSectionNode()
            : this(new UrlMappingsSection())
        {
        }

        public UrlMappingsSectionNode(UrlMappingsSection data)
            :base(data, data.UrlMappings.EmitClear)
        {
            foreach(UrlMapping urlMap in data.UrlMappings)
            {
                AddCollectionItem(new RWUrlMapping(urlMap));
            }
        }

        public override void Validate(IList<ValidationError> errors)
        {
            foreach (RWUrlMapping urlMapping in Collection)
            {
                if (!urlMapping.Url.StartsWith("~/"))
                {
                    errors.Add(new ValidationError(this, "Collection", "Only relative (~/) urls are allowed on property Url"));
                }
            }
            base.Validate(errors);
        }

        public override void CopyTo(object target)
        {
            UrlMappingsSection section = target as UrlMappingsSection;
            if (section != null)
            {
                section.UrlMappings.Clear();

                foreach (RWUrlMapping urlMapping in Collection)
                {
                    section.UrlMappings.Add(urlMapping.CreateElement());
                }
                section.UrlMappings.EmitClear = EmitClear;
            }
        }

        protected override void InitializeCommands(NakedNodeCommandRegistrar commandRegistrar)
        {
            
        }

        public class RWUrlMapping : CollectionItem<UrlMapping>
        {
            public RWUrlMapping()
            {
                base["url"] = "~/";
            }

            public RWUrlMapping(UrlMapping mapping)
                : base(mapping)
            {
            }

            internal string Url
            {
                get { return (string)base["url"]; }
            }

            internal string MappedUrl
            {
                get { return (string)base["mappedUrl"]; }
            }

            public override UrlMapping CreateElement()
            {
                return new UrlMapping(Url, MappedUrl);
            }
        }
    }

    
}
