using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Practices.EnterpriseLibrary.Configuration.Design;
using System.ComponentModel.Design;
using EnterpriseLibraryExtensions;

namespace EnterpriseLibraryExtensions
{
    public class ExtensionDesignManager : ConfigurationDesignManager, IGlobalEvents
    {
        private static List<ConfigurationNode> sitedNodes = new List<ConfigurationNode>();
        private static List<IConfigurationUIHierarchy> registeredHierarchies = new List<IConfigurationUIHierarchy>();
        private EventHandler<ConfigurationNodeChangedEventArgs> childAddedHandler;
        private EventHandler<HierarchySavedEventArgs> hierarchySavedHandler;
        private EventHandler<HierarchyAddedEventArgs> hierarchyAddedHandler;

        public ExtensionDesignManager()
        {
            childAddedHandler = new EventHandler<ConfigurationNodeChangedEventArgs>(ChildAdded);
            hierarchySavedHandler = new EventHandler<HierarchySavedEventArgs>(OnHierarchySaved);
            hierarchyAddedHandler = new EventHandler<HierarchyAddedEventArgs>(OnHierarchyAdded);
        }

        public override void Register(IServiceProvider serviceProvider)
        {
            RegisterService(serviceProvider);

            HostService.RegisterService(serviceProvider);
            ExtenderService.RegisterService(serviceProvider);
            ExtenderStorageService.RegisterService(serviceProvider);
            CurrentSelectionService.RegisterService(serviceProvider);
            MenuCommandService.RegisterService(serviceProvider);
            ConfigurationManagementService.RegisterService(serviceProvider);
            UIService2.RegisterService(serviceProvider);
            

            DesignerVerbLoader.DoLoad(serviceProvider);

            IConfigurationUIHierarchyService hierarchyService = ServiceHelper.GetUIHierarchyService(serviceProvider);
            
            foreach (IConfigurationUIHierarchy existingHierarchy in hierarchyService.GetAllHierarchies())
            {
                if (!registeredHierarchies.Contains(existingHierarchy))
                {
                    existingHierarchy.Saved += hierarchySavedHandler;
                    existingHierarchy.RootNode.ChildAdded += childAddedHandler;
                    registeredHierarchies.Add(existingHierarchy);
                }
            }
            hierarchyService.HierarchyAdded += new EventHandler<HierarchyAddedEventArgs>(hierarchyAddedHandler);
        }

        public void RegisterService(IServiceProvider serviceProvider)
        {
            if (serviceProvider.GetService(typeof(IGlobalEvents)) == null)
            {
                IServiceContainer serviceContainer = serviceProvider.GetService(typeof(IServiceContainer)) as IServiceContainer;
                if (serviceContainer != null)
                {
                    serviceContainer.AddService(typeof(IGlobalEvents), this);
                }
            }
        }

        private void OnHierarchyAdded(object sender, HierarchyAddedEventArgs e)
        {
            e.UIHierarchy.Saved += hierarchySavedHandler;
            e.UIHierarchy.RootNode.ChildAdded += childAddedHandler;
        }

        private void OnHierarchySaved(object sender, HierarchySavedEventArgs e)
        {
            if (HierarchySaved != null)
            {
                HierarchySaved(sender, e);
            }
        }

        private void ChildAdded(object sender, ConfigurationNodeChangedEventArgs e)
        {
            if (!sitedNodes.Contains(e.Node))
            {
                sitedNodes.Add(e.Node);
                CallSited(e.Node);

                if (NodeAdded != null)
                {
                    NodeAdded(sender, e);
                    foreach (ConfigurationNode childNode in e.Node.Nodes)
                    {
                        ChildAdded(this, new ConfigurationNodeChangedEventArgs(childNode, e.Node));
                    }
                }

                e.Node.ChildAdded += childAddedHandler;
            }
        }

        private void CallSited(ConfigurationNode node)
        {
            if (node is ISitedConfigurationNodeSink)
            {
                ((ISitedConfigurationNodeSink)node).Sited();
            }

            foreach (ConfigurationNode childNode in node.Nodes)
            {
                CallSited(childNode);
            }
        }

        #region IExtendedEvents Members

        public event EventHandler<ConfigurationNodeChangedEventArgs> NodeAdded;

        public event EventHandler<HierarchySavedEventArgs> HierarchySaved;

        #endregion
    }
}
