using System;
using System.Collections.Generic;
using System.Text;
using System.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Configuration.Design;
using System.ComponentModel.Design;
using Microsoft.Practices.EnterpriseLibrary.Common.Configuration;
using System.Reflection;

namespace EnterpriseLibraryExtensions
{
    class ConfigurationManagementService : IConfigurationManagementService
    {
        public string GetConfigurationFilePath(IConfigurationUIHierarchy uiHierarchy)
        {
            if (uiHierarchy.RootNode != null 
                && !string.IsNullOrEmpty(uiHierarchy.RootNode.ConfigurationFile))
            {
                return uiHierarchy.RootNode.ConfigurationFile;
            }

            return AppDomain.CurrentDomain.SetupInformation.ConfigurationFile;
        }

        public Configuration GetConfiguration(IConfigurationUIHierarchy uiHierarchy)
        {
            ExeConfigurationFileMap fileMap = new ExeConfigurationFileMap();
            fileMap.ExeConfigFilename = GetConfigurationFilePath(uiHierarchy);
            return ConfigurationManager.OpenMappedExeConfiguration(fileMap, ConfigurationUserLevel.None);
        }

        public void FlushConfigurationSourceCache(IConfigurationUIHierarchy uiHierarchy)
        {
            string configurationFilePath = GetConfigurationFilePath(uiHierarchy);
            MethodInfo resetImplMethod = typeof(FileConfigurationSource).GetMethod("ResetImplementation", BindingFlags.Static | BindingFlags.NonPublic);
            resetImplMethod.Invoke(null, new object[]{configurationFilePath, true});
         
        }

        

        public static void RegisterService(IServiceProvider serviceProvider)
        {
            if (serviceProvider.GetService(typeof(IConfigurationManagementService)) == null)
            {
                IServiceContainer serviceContainer = serviceProvider.GetService(typeof(IServiceContainer)) as IServiceContainer;
                if (serviceContainer != null)
                {
                    ConfigurationManagementService configurationMgmtService = new ConfigurationManagementService();
                    serviceContainer.AddService(typeof(IConfigurationManagementService), configurationMgmtService);
                }
            }
        }
    }

    public interface IConfigurationManagementService
    {
        string GetConfigurationFilePath(IConfigurationUIHierarchy uiHierarchy);
        Configuration GetConfiguration(IConfigurationUIHierarchy uiHierarchy);
        void FlushConfigurationSourceCache(IConfigurationUIHierarchy uiHierarchy);
    }
}
