using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Practices.EnterpriseLibrary.Configuration.Design;
using System.ComponentModel.Design;
using System.ComponentModel;
using System.Reflection;
using System.Windows.Forms;

namespace EnterpriseLibraryExtensions
{
    public class CurrentSelectionService : ICurrentSelectionService
    {
        IServiceProvider _svcProvider;
        IHostService _hostService;

        public CurrentSelectionService(IServiceProvider svcProvider, IHostService hostService)
        {
            _svcProvider = svcProvider;
            _hostService = hostService;
        }

        #region ICurrentSelectionService Members

        public ConfigurationNode CurrentlySelectedNode
        {
            get
            {

                IUIService uiService = (IUIService)_svcProvider.GetService(typeof(IUIService));
                if (uiService is IUIServiceProxy) uiService = ((IUIServiceProxy)uiService).ActualUIService;

                if (_hostService.CurrentHost == Host.Console)
                {
                    Type windowType = uiService.GetType();
                    FieldInfo propertyGridField = windowType.GetField("propertyGrid", BindingFlags.NonPublic | BindingFlags.Instance);
                    PropertyGrid propertyGrid = (PropertyGrid)propertyGridField.GetValue(uiService);

                    return propertyGrid.SelectedObject as ConfigurationNode;
                }
                if (_hostService.CurrentHost == Host.VisualStudio)
                {
                    FieldInfo propertyGridField = uiService.GetType().GetField("propertiesWindow", BindingFlags.NonPublic | BindingFlags.Instance);
                    object propertyGridWindow = propertyGridField.GetValue(uiService);
                    FieldInfo selectedItemsField = propertyGridWindow.GetType().GetField("selection", BindingFlags.NonPublic | BindingFlags.Instance);
                    System.Collections.ArrayList list = (System.Collections.ArrayList)selectedItemsField.GetValue(propertyGridWindow);
                    if (list.Count > 0)
                    {
                        return list[0] as ConfigurationNode;
                    }
                    return null;
                }

                return null;
            }
        }

        #endregion

        public static void RegisterService(IServiceProvider serviceProvider)
        {
            if (serviceProvider.GetService(typeof(ICurrentSelectionService)) == null)
            {
                IServiceContainer serviceContainer = serviceProvider.GetService(typeof(IServiceContainer)) as IServiceContainer;
                if (serviceContainer != null)
                {
                    IHostService hostService = (IHostService)serviceProvider.GetService(typeof(IHostService));
                    CurrentSelectionService selectionService = new CurrentSelectionService(serviceContainer, hostService);
                    serviceContainer.AddService(typeof(ICurrentSelectionService), selectionService);
                }
            }
        }
    }

    public interface ICurrentSelectionService
    {
        ConfigurationNode CurrentlySelectedNode { get; }
    }
}
