using System.Collections.Generic;
using System.ComponentModel.Design;
using System.ComponentModel;
using System;
using Microsoft.Practices.EnterpriseLibrary.Configuration.Design;

namespace EnterpriseLibraryExtensions
{
    class ExtenderService : IExtenderListService, IExtenderProviderService
    {
        List<IExtenderProvider> extenderProviders = new List<IExtenderProvider>();

        void IExtenderProviderService.AddExtenderProvider(IExtenderProvider provider)
        {
            extenderProviders.Add(provider);
        }

        void IExtenderProviderService.RemoveExtenderProvider(IExtenderProvider provider)
        {
            extenderProviders.Remove(provider);
        }

        IExtenderProvider[] IExtenderListService.GetExtenderProviders()
        {
            return extenderProviders.ToArray();
        }

        public static void RegisterService(IServiceProvider serviceProvider)
        {
            IServiceContainer serviceContainer = (IServiceContainer)serviceProvider.GetService(typeof(IServiceContainer));
            if (serviceContainer.GetService(typeof(IExtenderProviderService)) == null)
            {
                ExtenderService extenderService = new ExtenderService();
                serviceContainer.AddService(typeof(IExtenderProviderService), extenderService);
                serviceContainer.AddService(typeof(IExtenderListService), extenderService);
            }

            IConfigurationUIHierarchy currentHierarchy = ServiceHelper.GetCurrentHierarchy(serviceProvider);
            if (currentHierarchy.RootNode != null)
            {
                currentHierarchy.RootNode.ChildAdded += new EventHandler<ConfigurationNodeChangedEventArgs>(RegisterProvidersChildAdded);
            }
        }

        private static void RegisterProvidersChildAdded(object sender, ConfigurationNodeChangedEventArgs e)
        {
            AddExtenderProviders(e.Node);
            e.Node.ChildAdded += new EventHandler<ConfigurationNodeChangedEventArgs>(RegisterProvidersChildAdded);
        }

        private static void AddExtenderProviders(ConfigurationNode node)
        {
            if (node is IExtenderProvider)
            {
                IExtenderProviderService extenderProviderSvc = node.Site.GetService(typeof(IExtenderProviderService)) as IExtenderProviderService;
                if (extenderProviderSvc != null)
                {
                    extenderProviderSvc.AddExtenderProvider(node as IExtenderProvider);
                }
            }

            foreach (ConfigurationNode childNode in node.Nodes)
            {
                AddExtenderProviders(childNode);
            }
        }
    }
}
