using System;
using System.Collections.Generic;
using System.Text;
using System.ComponentModel.Design;
using Microsoft.Practices.EnterpriseLibrary.Configuration.Design;

namespace EnterpriseLibraryExtensions
{
    public class MenuCommandService : IMenuCommandService, IConfigDesignerVerbCollection
    {
        IServiceProvider _serviceProvider;
        List<IConfigDesignerVerb> _configDesignerVerbs = new List<IConfigDesignerVerb>();
        List<DesignerVerb> _normalVerbs = new List<DesignerVerb>();


        public MenuCommandService(IServiceProvider serviceProvider)
        {
            _serviceProvider = serviceProvider;
        }

        #region IMenuCommandService Members

        public void AddCommand(MenuCommand command)
        {
            
        }

        public void AddVerb(DesignerVerb verb)
        {
            _normalVerbs.Add(verb);
        }

        public MenuCommand FindCommand(CommandID commandID)
        {
            return null;
        }

        public bool GlobalInvoke(CommandID commandID)
        {
            return false;
        }

        public void RemoveCommand(MenuCommand command)
        {
            
        }

        public void RemoveVerb(DesignerVerb verb)
        {
            _normalVerbs.Remove(verb);
        }

        public void ShowContextMenu(CommandID menuID, int x, int y)
        {
            
        }

        public DesignerVerbCollection Verbs
        {
            get
            {
                DesignerVerbCollection verbCollection = new DesignerVerbCollection();
                IConfigurationUIHierarchyService hierarchyService = (IConfigurationUIHierarchyService)_serviceProvider.GetService(typeof(IConfigurationUIHierarchyService));
                ICurrentSelectionService currentSelection = (ICurrentSelectionService)_serviceProvider.GetService(typeof(ICurrentSelectionService));

                ConfigurationNode node = currentSelection.CurrentlySelectedNode;

                foreach (IConfigDesignerVerb configDesginerVerb in _configDesignerVerbs)
                {
                    foreach(DesignerVerb designVerb in configDesginerVerb.VerbsForCofigurationNode(node))
                    {
                        verbCollection.Add(designVerb);
                    }
                }

                return verbCollection;
            }
        }

        #endregion

        public static void RegisterService(IServiceProvider serviceProvider)
        {
            if (serviceProvider.GetService(typeof(IMenuCommandService)) == null)
            {
                IServiceContainer serviceContainer = serviceProvider.GetService(typeof(IServiceContainer)) as IServiceContainer;
                if (serviceContainer != null)
                {
                    MenuCommandService menuService = new MenuCommandService(serviceProvider);
                    serviceContainer.AddService(typeof(IMenuCommandService), menuService);
                    serviceContainer.AddService(typeof(IConfigDesignerVerbCollection), menuService);
                    
                }
            }
        }

        #region IConfigDesignerVerbCollection Members

        public void Add(IConfigDesignerVerb verb)
        {
            _configDesignerVerbs.Add(verb);
        }

        public void Remove(IConfigDesignerVerb verb)
        {
            _configDesignerVerbs.Remove(verb);
        }

        #endregion
    }
}
