using System;
using System.Collections.Generic;
using System.Text;
using System.ComponentModel.Design;
using Microsoft.Practices.EnterpriseLibrary.Configuration.Design;
using EnterpriseLibraryExtensions;

[assembly:ConfigDesignVerbExportAttribute(typeof(NodeInformationVerb))]

namespace EnterpriseLibraryExtensions
{
    public class NodeInformationVerb : IConfigDesignerVerb
    {

        public List<DesignerVerb> VerbsForCofigurationNode(ConfigurationNode node)
        {
            List<DesignerVerb> verbs = new List<DesignerVerb>();
            if (node != null)
            {
                object[] nodeInformationAttributes = node.GetType().GetCustomAttributes(typeof(NodeInformationAttribute), false);
                if (nodeInformationAttributes != null && nodeInformationAttributes.Length > 0)
                {
                    verbs.Add(new VerbContainer((IUIService)node.Site.GetService(typeof(IUIService)), (NodeInformationAttribute)nodeInformationAttributes[0]).TheVerb);
                }
            }

            return verbs;
        }

        private class VerbContainer
        {
            IUIService _uiService;
            NodeInformationAttribute _nodeInformation;
            DesignerVerb _verb;

            public VerbContainer(IUIService uiService, NodeInformationAttribute nodeInformation)
            {
                _uiService = uiService;
                _nodeInformation = nodeInformation;
                _verb = new DesignerVerb("Information about this node", new EventHandler(DoExecute));
            }

            public DesignerVerb TheVerb
            {
                get { return _verb; }
            }

            private void DoExecute(object sender, EventArgs e)
            {
                using (NodeInformationUI informationUI = new NodeInformationUI(_nodeInformation))
                {
                    informationUI.ShowDialog();
                }
            }
        }
    }
}
