using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Practices.EnterpriseLibrary.Configuration.Design;
using System.Reflection;
using System.ComponentModel.Design;

namespace EnterpriseLibraryExtensions
{
    interface IUIServiceProxy
    {
        IUIService ActualUIService { get; }
    }

    class UIService2 : IUIService, IUIServiceProxy
    {
        IUIService _actualService;
        ICurrentSelectionService _selectionService;
        IHostService _hostService;

        public UIService2(IUIService actualService, ICurrentSelectionService selectionService, IHostService hostService)
        {
            _hostService = hostService;
            _actualService = actualService;
            _selectionService = selectionService;
        }

        #region IUIService Members

        public void ActivateNode(ConfigurationNode node)
        {
            _actualService.ActivateNode(node);
        }

        public void BeginUpdate()
        {
            _actualService.BeginUpdate();
        }

        public void ClearErrorDisplay()
        {
            _actualService.ClearErrorDisplay();
        }

        public void DisplayErrorLog(IErrorLogService errorLogService)
        {
            _actualService.DisplayErrorLog(errorLogService);
        }

        public void EndUpdate()
        {
            _actualService.EndUpdate();
        }

        public bool IsDirty(IConfigurationUIHierarchy hierarchy)
        {
            return _actualService.IsDirty(hierarchy);
        }

        public System.Windows.Forms.IWin32Window OwnerWindow
        {
            get { return _actualService.OwnerWindow; }
        }

        public void RefreshPropertyGrid()
        {
            if (_hostService.CurrentHost == Host.VisualStudio)
            {
                object currentSelection = _selectionService.CurrentlySelectedNode;

                FieldInfo propertyGridField = _actualService.GetType().GetField("propertiesWindow", BindingFlags.NonPublic | BindingFlags.Instance);
                object propertyGridWindow = propertyGridField.GetValue(_actualService);

                MethodInfo clearMethod = propertyGridWindow.GetType().GetMethod("ClearSelection", BindingFlags.Public | BindingFlags.Instance);
                clearMethod.Invoke(propertyGridWindow, new object[0]);

                MethodInfo setSelectionMethod = propertyGridWindow.GetType().GetMethod("SetSelection", BindingFlags.Public | BindingFlags.Instance);
                setSelectionMethod.Invoke(propertyGridWindow, new object[] { currentSelection });
            }
            else
            {
                _actualService.RefreshPropertyGrid();
            }
        }

        public void SetStatus(string status)
        {
            _actualService.SetStatus(status);
        }

        public void SetUIDirty(IConfigurationUIHierarchy hierarchy)
        {
            _actualService.SetUIDirty(hierarchy);
        }

        public void ShowError(string message, string caption)
        {
            _actualService.ShowError(message, caption);
        }

        public void ShowError(Exception e, string message, string caption)
        {
            _actualService.ShowError(e, message, caption);
        }

        public System.Windows.Forms.DialogResult ShowOpenDialog(System.Windows.Forms.OpenFileDialog dialog)
        {
            return _actualService.ShowOpenDialog(dialog);
        }

        public System.Windows.Forms.DialogResult ShowSaveDialog(System.Windows.Forms.SaveFileDialog dialog)
        {
            return _actualService.ShowSaveDialog(dialog); ;
        }

        #endregion

        #region IUIServiceProxy Members

        public IUIService ActualUIService
        {
            get { return _actualService; }
        }

        #endregion

        #region IUIService Members


        public void ShowError(string message)
        {
            _actualService.ShowError(message);
        }

        public void ShowError(Exception e, string message)
        {
            _actualService.ShowError(e, message);
        }

        public void ShowError(Exception e)
        {
            _actualService.ShowError(e);
        }

        public System.Windows.Forms.DialogResult ShowMessage(string message, string caption, System.Windows.Forms.MessageBoxButtons buttons)
        {
            return _actualService.ShowMessage(message, caption, buttons);
        }

        public void ShowMessage(string message, string caption)
        {
            _actualService.ShowMessage(message, caption);
        }

        public void ShowMessage(string message)
        {
            _actualService.ShowMessage(message);
        }

        #endregion

        public static void RegisterService(IServiceProvider serviceProvider)
        {
            if (serviceProvider.GetService(typeof(IUIService)) as UIService2 == null)
            {
                IServiceContainer serviceContainer = serviceProvider.GetService(typeof(IServiceContainer)) as IServiceContainer;
                if (serviceContainer != null)
                {
                    IHostService hostService = (IHostService)serviceProvider.GetService(typeof(IHostService));
                    ICurrentSelectionService selectionService = (ICurrentSelectionService)serviceProvider.GetService(typeof(ICurrentSelectionService));
                    IUIService actualService = (IUIService)serviceProvider.GetService(typeof(IUIService));
                    serviceContainer.RemoveService(typeof(IUIService));
                    serviceContainer.AddService(typeof(IUIService), new UIService2(actualService, selectionService, hostService));
                }
            }
        }
    }
}
